const fs = require('node:fs')
const path = require('node:path')
const fg = require('fast-glob');
const chokidar = require('chokidar')

const postcss = require('postcss')
const postcssModules = require('postcss-modules')
const autoprefixer = require('autoprefixer')

const { src, dist } = require('./config');
const cssGlob = `${src}/styles/*.css`

async function watchCSS() {
  chokidar.watch(cssGlob)
    .on('add', transformCSS)
    .on('change', transformCSS)
    .on('unlink', transformCSS)

  async function transformCSS() {
    const cssFiles = await fg(cssGlob);

    const moduleTransformations = {}
    const processor = postcss([
      autoprefixer,
      postcssModules({
        getJSON: function (fileName, classes) {
          const moduleName = path.basename(fileName, '.css')
          moduleTransformations[moduleName] = {
            classes: classes
          }
        }
      })
    ])

    for (const fileName of cssFiles) {
      const moduleSource = (await fs.promises.readFile(fileName)).toString('utf8')
      const moduleName = path.basename(fileName, '.css')
      if (!moduleName.match(/[a-zA-Z]+/)) {
        console.error(`Module name ${moduleName}.css is not allowed (only letters)`)
        continue
      }

      let result
      try {
        result = await processor.process(moduleSource, {
          from: fileName,
          to: fileName
        })
      } catch (error) {
        if(error.name !== 'CssSyntaxError')
          throw error

        console.error(error.message + '\n' + error.showSourceCode())
        continue
      }

      result.warnings().forEach(console.error.bind(console))

      moduleTransformations[moduleName].text = result.css
    }

    let typeDef = `
      declare const styleComponent: () => React.ReactElement
      export default styleComponent
    `

    const styleText = Object.values(moduleTransformations)
      .map(module => module.text.split('\n').join(' '))
      .join(' ')
      .replace(/ +/g, ' ')

    let moduleDef = `
      export default function Styles() {
        return (
          React.createElement('style', null, '${styleText.replaceAll("'", "\\'")}')
        )
      }
    `

    for (const module in moduleTransformations) {
      typeDef += `
        export const ${module}: {
          ${Object.keys(moduleTransformations[module].classes)
            .map(className => `${className}: string,`)
            .join('\n')
          }
        }
      `

      moduleDef += `
        export const ${module} = ${JSON.stringify(moduleTransformations[module].classes)}
      `
    }

    console.log(styleText)


    typeDef = typeDef.replace(/^ +/gm, '')
    moduleDef = moduleDef.replace(/^ +/gm, '')

    await fs.promises.writeFile(`${src}/styles/.generated.d.ts`, typeDef)
    await fs.promises.writeFile(`${dist}/styles.js`, moduleDef)
  }
}

watchCSS()
